"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.register = void 0;
function register(ts, context) {
    return {
        getRootFileNames,
        emit,
        getSyntacticDiagnostics,
        getSemanticDiagnostics,
        getGlobalDiagnostics,
        getBindAndCheckDiagnostics,
    };
    function getRootFileNames() {
        return getProgram()?.getRootFileNames().filter(fileName => context.getTsLsHost().fileExists?.(fileName));
    }
    // for vue-tsc --noEmit --watch
    function getBindAndCheckDiagnostics(sourceFile, cancellationToken) {
        return getSourceFileDiagnosticsWorker(sourceFile, cancellationToken, 'getBindAndCheckDiagnostics');
    }
    // for vue-tsc --noEmit
    function getSyntacticDiagnostics(sourceFile, cancellationToken) {
        return getSourceFileDiagnosticsWorker(sourceFile, cancellationToken, 'getSyntacticDiagnostics');
    }
    function getSemanticDiagnostics(sourceFile, cancellationToken) {
        return getSourceFileDiagnosticsWorker(sourceFile, cancellationToken, 'getSemanticDiagnostics');
    }
    function getSourceFileDiagnosticsWorker(sourceFile, cancellationToken, api) {
        if (sourceFile) {
            const mapped = context.vueFiles.fromEmbeddedFileName(sourceFile.fileName);
            if (mapped) {
                let results = [];
                const embeddeds = mapped.vueFile.getAllEmbeddeds();
                for (const embedded of embeddeds) {
                    const isTsFile = embedded.file.fileName.endsWith('.js') ||
                        embedded.file.fileName.endsWith('.ts') ||
                        embedded.file.fileName.endsWith('.jsx') ||
                        embedded.file.fileName.endsWith('.tsx');
                    if (!isTsFile || !embedded.file.capabilities.diagnostics)
                        continue;
                    const program = getProgram();
                    const embeddedSourceFile = program?.getSourceFile(embedded.file.fileName);
                    if (embeddedSourceFile) {
                        const errors = transformDiagnostics(program?.[api](embeddedSourceFile, cancellationToken) ?? []);
                        results = results.concat(errors);
                    }
                }
                return results;
            }
            else {
                return getProgram()?.[api](sourceFile, cancellationToken) ?? [];
            }
        }
        return transformDiagnostics(getProgram()?.[api](sourceFile, cancellationToken) ?? []);
    }
    function getGlobalDiagnostics(cancellationToken) {
        return transformDiagnostics(getProgram()?.getGlobalDiagnostics(cancellationToken) ?? []);
    }
    function emit(targetSourceFile, _writeFile, cancellationToken, emitOnlyDtsFiles, customTransformers) {
        const scriptResult = getProgram().emit(targetSourceFile, (context.vueLsHost.writeFile ?? ts.sys.writeFile), cancellationToken, emitOnlyDtsFiles, customTransformers);
        return {
            emitSkipped: scriptResult.emitSkipped,
            emittedFiles: scriptResult.emittedFiles,
            diagnostics: transformDiagnostics(scriptResult.diagnostics),
        };
    }
    function getProgram() {
        return context.getTsLs().getProgram();
    }
    // transform
    function transformDiagnostics(diagnostics) {
        const result = [];
        for (const diagnostic of diagnostics) {
            if (diagnostic.file !== undefined
                && diagnostic.start !== undefined
                && diagnostic.length !== undefined) {
                for (const tsOrVueLoc of context.vueFiles.fromEmbeddedLocation(diagnostic.file.fileName, diagnostic.start, diagnostic.start + diagnostic.length, data => !!data.capabilities.diagnostic)) {
                    if (!context.vueLsHost.fileExists?.(tsOrVueLoc.fileName))
                        continue;
                    let file = tsOrVueLoc.fileName === diagnostic.file.fileName
                        ? diagnostic.file
                        : undefined;
                    if (!file) {
                        let docText = tsOrVueLoc.mapped?.vueFile.getContent();
                        if (docText === undefined) {
                            const snapshot = context.vueLsHost.getScriptSnapshot(tsOrVueLoc.fileName);
                            if (snapshot) {
                                docText = snapshot.getText(0, snapshot.getLength());
                            }
                        }
                        else {
                            file = ts.createSourceFile(tsOrVueLoc.fileName, docText, tsOrVueLoc.fileName.endsWith('.vue') ? ts.ScriptTarget.JSON : ts.ScriptTarget.Latest);
                        }
                    }
                    const newDiagnostic = {
                        ...diagnostic,
                        file,
                        start: tsOrVueLoc.range.start,
                        length: tsOrVueLoc.range.end - tsOrVueLoc.range.start,
                    };
                    const relatedInformation = diagnostic.relatedInformation;
                    if (relatedInformation) {
                        newDiagnostic.relatedInformation = transformDiagnostics(relatedInformation);
                    }
                    result.push(newDiagnostic);
                }
            }
            else if (diagnostic.file === undefined) {
                result.push(diagnostic);
            }
        }
        return result;
    }
}
exports.register = register;
//# sourceMappingURL=apis.js.map